/*
 * Copyright (c) 2000-2018, 达梦数据库有限公司.
 * All rights reserved.
 */
package dm

import (
	"dmdb_exporter/dm/util"
	"strconv"
	"time"
)

var DB2G db2g

type db2g struct {
}

func (DB2G db2g) processVarchar2(bytes []byte, prec int) []byte {
	rbytes := make([]byte, prec)
	copy(rbytes[:len(bytes)], bytes[:])
	for i := len(bytes); i < len(rbytes); i++ {
		rbytes[i] = ' '
	}
	return rbytes
}

func (DB2G db2g) charToString(bytes []byte, column *column, conn *DmConnection) string {
	if column.colType == VARCHAR2 {
		bytes = DB2G.processVarchar2(bytes, int(column.prec))
	} else if column.colType == CLOB {

	}
	return util.Dm_build_586.Dm_build_828(bytes, conn.serverEncoding)
}

func (DB2G db2g) charToFloat64(bytes []byte, column *column, conn *DmConnection) (float64, error) {
	str := DB2G.charToString(bytes, column, conn)
	val, err := strconv.ParseFloat(str, 64)
	if err != nil {
		return 0, ECGO_DATA_CONVERTION_ERROR.throw()
	}

	return val, nil
}

func (DB2G db2g) BinaryToInt64(bytes []byte, column *column, conn *DmConnection) (int64, error) {
	if column.colType == BLOB {

	}
	var n, b int64 = 0, 0

	startIndex := 0
	var length int
	if len(bytes) > 8 {
		length = 8
		for j := 0; j < len(bytes)-8; j++ {
			if bytes[j] != 0 {
				return 0, ECGO_DATA_CONVERTION_ERROR.throw()
			}

			startIndex = len(bytes) - 8
			length = 8
		}
	} else {
		length = len(bytes)
	}

	for j := startIndex; j < startIndex+length; j++ {
		b = int64(0xff & bytes[j])
		n = b | (n << 8)
	}

	return n, nil
}

func (DB2G db2g) toBytes(bytes []byte, column *column, conn *DmConnection) ([]byte, error) {
	retBytes := util.Dm_build_586.Dm_build_735(bytes, 0, len(bytes))
	switch column.colType {
	case CLOB:
		clob := newClobFromDB(retBytes, conn, column, true)
		str, err := clob.getSubString(1, int32(clob.length))
		if err != nil {
			return nil, err
		}

		return util.Dm_build_586.Dm_build_793(str, conn.getServerEncoding()), nil
	case BLOB:
		blob := newBlobFromDB(retBytes, conn, column, true)
		bs, err := blob.getBytes(1, int32(blob.length))
		if err != nil {
			return nil, err
		}

		return bs, nil
	}
	return nil, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toString(bytes []byte, column *column, conn *DmConnection) string {
	switch column.colType {
	case CHAR, VARCHAR, VARCHAR2:
		return DB2G.charToString(bytes, column, conn)
	case BIT, BOOLEAN, TINYINT:
		return strconv.FormatInt(int64(bytes[0]), 10)
	case SMALLINT:
		return strconv.FormatInt(int64(util.Dm_build_586.Dm_build_800(bytes)), 10)
	case INT:
		return strconv.FormatInt(int64(util.Dm_build_586.Dm_build_803(bytes)), 10)
	case BIGINT:
		return strconv.FormatInt(int64(util.Dm_build_586.Dm_build_806(bytes)), 10)
	case REAL:
		return strconv.FormatFloat(float64(util.Dm_build_586.Dm_build_809(bytes)), 'f', -1, 32)
	case DOUBLE:
		return strconv.FormatFloat(float64(util.Dm_build_586.Dm_build_812(bytes)), 'f', -1, 64)
	case DECIMAL:

	case BINARY, VARBINARY:
		util.StringUtil.BytesToHexString(bytes, false)
	case BLOB:

	case CLOB:

	case DATE:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		if conn.OracleDateFormat != "" {
			return dtToStringByOracleFormat(dt, conn.OracleDateFormat, int(conn.OracleDateLanguage))
		}
	case TIME:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		if conn.OracleTimeFormat != "" {
			return dtToStringByOracleFormat(dt, conn.OracleTimeFormat, int(conn.OracleDateLanguage))
		}
	case DATETIME:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		if conn.OracleTimestampFormat != "" {
			return dtToStringByOracleFormat(dt, conn.OracleTimestampFormat, int(conn.OracleDateLanguage))
		}
	case TIME_TZ:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		if conn.OracleTimeTZFormat != "" {
			return dtToStringByOracleFormat(dt, conn.OracleTimeTZFormat, int(conn.OracleDateLanguage))
		}
	case DATETIME_TZ:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		if conn.OracleTimestampTZFormat != "" {
			return dtToStringByOracleFormat(dt, conn.OracleTimestampTZFormat, int(conn.OracleDateLanguage))
		}
	case INTERVAL_DT:
		return newDmIntervalDTByBytes(bytes).String()
	case INTERVAL_YM:
		return newDmIntervalYMByBytes(bytes).String()
	case ARRAY:

	case SARRAY:

	case CLASS:

	case PLTYPE_RECORD:

	}
	return ""
}

func (DB2G db2g) toBool(bytes []byte, column *column, conn *DmConnection) (bool, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		return bytes[0] != 0, nil
	case SMALLINT:
		return util.Dm_build_586.Dm_build_681(bytes, 0) != 0, nil
	case INT:
		return util.Dm_build_586.Dm_build_686(bytes, 0) != 0, nil
	case BIGINT:
		return util.Dm_build_586.Dm_build_691(bytes, 0) != 0, nil
	case REAL:
		return util.Dm_build_586.Dm_build_696(bytes, 0) != 0, nil
	case DOUBLE:
		return util.Dm_build_586.Dm_build_700(bytes, 0) != 0, nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		return G2DB.toBool(DB2G.charToString(bytes, column, conn))
	}

	return false, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toByte(bytes []byte, column *column, conn *DmConnection) (byte, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:

		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		} else {
			return bytes[0], nil
		}
	case SMALLINT:
		tval := util.Dm_build_586.Dm_build_681(bytes, 0)
		if tval < int16(BYTE_MIN) || tval > int16(BYTE_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return byte(tval), nil
	case INT:
		tval := util.Dm_build_586.Dm_build_686(bytes, 0)
		if tval < int32(BYTE_MIN) || tval > int32(BYTE_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return byte(tval), nil
	case BIGINT:
		tval := util.Dm_build_586.Dm_build_691(bytes, 0)
		if tval < int64(BYTE_MIN) || tval > int64(BYTE_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return byte(tval), nil
	case REAL:
		tval := util.Dm_build_586.Dm_build_696(bytes, 0)
		if tval < float32(BYTE_MIN) || tval > float32(BYTE_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return byte(tval), nil
	case DOUBLE:
		tval := util.Dm_build_586.Dm_build_700(bytes, 0)
		if tval < float64(BYTE_MIN) || tval > float64(BYTE_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return byte(tval), nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if tval < float64(BYTE_MIN) || tval > float64(BYTE_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return byte(tval), nil
	case BINARY, VARBINARY, BLOB:
		{
			tval, err := DB2G.BinaryToInt64(bytes, column, conn)
			if err != nil {
				return 0, err
			}

			if tval < int64(BYTE_MIN) || tval > int64(BYTE_MAX) {
				return 0, ECGO_DATA_OVERFLOW.throw()
			}
			return byte(tval), nil
		}
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toInt8(bytes []byte, column *column, conn *DmConnection) (int8, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}

		return int8(bytes[0]), nil
	case SMALLINT:
		tval := util.Dm_build_586.Dm_build_681(bytes, 0)
		if tval < int16(INT8_MIN) || tval < int16(INT8_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int8(tval), nil
	case INT:

		tval := util.Dm_build_586.Dm_build_686(bytes, 0)
		if tval < int32(INT8_MIN) || tval > int32(INT8_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int8(tval), nil
	case BIGINT:
		tval := util.Dm_build_586.Dm_build_691(bytes, 0)
		if tval < int64(INT8_MIN) || tval > int64(INT8_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int8(tval), nil
	case REAL:
		tval := util.Dm_build_586.Dm_build_696(bytes, 0)
		if tval < float32(INT8_MIN) || tval > float32(INT8_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int8(tval), nil
	case DOUBLE:
		tval := util.Dm_build_586.Dm_build_700(bytes, 0)
		if tval < float64(INT8_MIN) || tval > float64(INT8_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int8(tval), nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if tval < float64(INT8_MIN) || tval > float64(INT8_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int8(tval), nil
	case BINARY, VARBINARY, BLOB:
		{
			tval, err := DB2G.BinaryToInt64(bytes, column, conn)
			if err != nil {
				return 0, err
			}

			if tval < int64(INT8_MIN) || tval > int64(INT8_MAX) {
				return 0, ECGO_DATA_OVERFLOW.throw()
			}
			return int8(tval), nil
		}
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toInt16(bytes []byte, column *column, conn *DmConnection) (int16, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}

		return int16(bytes[0]), nil
	case SMALLINT:
		return util.Dm_build_586.Dm_build_681(bytes, 0), nil
	case INT:

		tval := util.Dm_build_586.Dm_build_686(bytes, 0)
		if tval < int32(INT16_MIN) || tval > int32(INT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int16(tval), nil
	case BIGINT:
		tval := util.Dm_build_586.Dm_build_691(bytes, 0)
		if tval < int64(INT16_MIN) || tval > int64(INT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int16(tval), nil
	case REAL:
		tval := util.Dm_build_586.Dm_build_696(bytes, 0)
		if tval < float32(INT16_MIN) || tval > float32(INT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int16(tval), nil
	case DOUBLE:
		tval := util.Dm_build_586.Dm_build_700(bytes, 0)
		if tval < float64(INT16_MIN) || tval > float64(INT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int16(tval), nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if tval < float64(INT16_MIN) || tval > float64(INT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int16(tval), nil
	case BINARY, VARBINARY, BLOB:
		{
			tval, err := DB2G.BinaryToInt64(bytes, column, conn)
			if err != nil {
				return 0, err
			}

			if tval < int64(INT16_MIN) || tval > int64(INT16_MAX) {
				return 0, ECGO_DATA_OVERFLOW.throw()
			}
			return int16(tval), nil
		}
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toUInt16(bytes []byte, column *column, conn *DmConnection) (uint16, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}

		return uint16(bytes[0]), nil
	case SMALLINT:
		return uint16(util.Dm_build_586.Dm_build_681(bytes, 0)), nil
	case INT:
		tval := util.Dm_build_586.Dm_build_686(bytes, 0)
		if tval < int32(UINT16_MIN) || tval > int32(UINT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint16(tval), nil
	case BIGINT:
		tval := util.Dm_build_586.Dm_build_691(bytes, 0)
		if tval < int64(UINT16_MIN) || tval > int64(UINT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint16(tval), nil
	case REAL:
		tval := util.Dm_build_586.Dm_build_696(bytes, 0)
		if tval < float32(UINT16_MIN) || tval > float32(UINT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint16(tval), nil
	case DOUBLE:
		tval := util.Dm_build_586.Dm_build_700(bytes, 0)
		if tval < float64(UINT16_MIN) || tval > float64(UINT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint16(tval), nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if tval < float64(UINT16_MIN) || tval > float64(UINT16_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint16(tval), nil
	case BINARY, VARBINARY, BLOB:
		{
			tval, err := DB2G.BinaryToInt64(bytes, column, conn)
			if err != nil {
				return 0, err
			}

			if tval < int64(UINT16_MIN) || tval > int64(UINT16_MAX) {
				return 0, ECGO_DATA_OVERFLOW.throw()
			}
			return uint16(tval), nil
		}
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toInt32(bytes []byte, column *column, conn *DmConnection) (int32, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}

		return int32(bytes[0]), nil
	case SMALLINT:
		return util.Dm_build_586.Dm_build_686(bytes, 0), nil
	case INT:
		return util.Dm_build_586.Dm_build_686(bytes, 0), nil
	case BIGINT:
		tval := util.Dm_build_586.Dm_build_691(bytes, 0)
		if tval < int64(INT32_MIN) || tval > int64(INT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int32(tval), nil
	case REAL:
		tval := util.Dm_build_586.Dm_build_696(bytes, 0)
		if tval < float32(INT32_MIN) || tval > float32(INT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int32(tval), nil
	case DOUBLE:
		tval := util.Dm_build_586.Dm_build_700(bytes, 0)
		if tval < float64(INT32_MIN) || tval > float64(INT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int32(tval), nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if tval < float64(INT32_MIN) || tval > float64(INT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int32(tval), nil
	case BINARY, VARBINARY, BLOB:
		{
			tval, err := DB2G.BinaryToInt64(bytes, column, conn)
			if err != nil {
				return 0, err
			}

			if tval < int64(INT32_MIN) || tval > int64(INT32_MAX) {
				return 0, ECGO_DATA_OVERFLOW.throw()
			}
			return int32(tval), nil
		}
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toUInt32(bytes []byte, column *column, conn *DmConnection) (uint32, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}

		return uint32(bytes[0]), nil
	case SMALLINT:
		return uint32(util.Dm_build_586.Dm_build_686(bytes, 0)), nil
	case INT:
		return uint32(util.Dm_build_586.Dm_build_686(bytes, 0)), nil
	case BIGINT:
		tval := util.Dm_build_586.Dm_build_691(bytes, 0)
		if tval < int64(UINT32_MIN) || tval > int64(UINT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint32(tval), nil
	case REAL:
		tval := util.Dm_build_586.Dm_build_696(bytes, 0)
		if tval < float32(UINT32_MIN) || tval > float32(UINT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint32(tval), nil
	case DOUBLE:
		tval := util.Dm_build_586.Dm_build_700(bytes, 0)
		if tval < float64(UINT32_MIN) || tval > float64(UINT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint32(tval), nil
	case DECIMAL:

	case CHAR, VARCHAR, VARCHAR2, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if tval < float64(UINT32_MIN) || tval > float64(UINT32_MAX) {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint32(tval), nil
	case BINARY, VARBINARY, BLOB:
		{
			tval, err := DB2G.BinaryToInt64(bytes, column, conn)
			if err != nil {
				return 0, err
			}

			if tval < int64(UINT32_MIN) || tval > int64(UINT32_MAX) {
				return 0, ECGO_DATA_OVERFLOW.throw()
			}
			return uint32(tval), nil
		}
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toInt64(bytes []byte, column *column, conn *DmConnection) (int64, error) {
	switch column.colType {
	case BOOLEAN, BIT, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return int64(0), nil
		} else {
			return int64(bytes[0]), nil
		}
	case SMALLINT:
		return int64(util.Dm_build_586.Dm_build_800(bytes)), nil
	case INT:
		return int64(util.Dm_build_586.Dm_build_803(bytes)), nil
	case BIGINT:
		return int64(util.Dm_build_586.Dm_build_806(bytes)), nil
	case REAL:
		return int64(util.Dm_build_586.Dm_build_809(bytes)), nil
	case DOUBLE:
		return int64(util.Dm_build_586.Dm_build_812(bytes)), nil

	case CHAR, VARCHAR2, VARCHAR, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if int64(tval) < INT64_MIN || int64(tval) > INT64_MAX {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return int64(tval), nil
	case BINARY, VARBINARY, BLOB:
		tval, err := DB2G.BinaryToInt64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		return tval, nil
	}
	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toUInt64(bytes []byte, column *column, conn *DmConnection) (uint64, error) {
	switch column.colType {
	case BOOLEAN, BIT, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return uint64(0), nil
		} else {
			return uint64(bytes[0]), nil
		}
	case SMALLINT:
		return uint64(util.Dm_build_586.Dm_build_800(bytes)), nil
	case INT:
		return uint64(util.Dm_build_586.Dm_build_803(bytes)), nil
	case BIGINT:
		return uint64(util.Dm_build_586.Dm_build_806(bytes)), nil
	case REAL:
		return uint64(util.Dm_build_586.Dm_build_809(bytes)), nil
	case DOUBLE:
		return uint64(util.Dm_build_586.Dm_build_812(bytes)), nil

	case CHAR, VARCHAR2, VARCHAR, CLOB:
		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		if uint64(tval) < UINT64_MIN || uint64(tval) > UINT64_MAX {
			return 0, ECGO_DATA_OVERFLOW.throw()
		}
		return uint64(tval), nil
	case BINARY, VARBINARY, BLOB:
		tval, err := DB2G.BinaryToInt64(bytes, column, conn)
		if err != nil {
			return 0, err
		}

		return uint64(tval), nil
	}
	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toInt(bytes []byte, column *column, conn *DmConnection) (int, error) {
	if strconv.IntSize == 32 {
		tmp, err := DB2G.toInt32(bytes, column, conn)
		return int(tmp), err
	} else {
		tmp, err := DB2G.toInt64(bytes, column, conn)
		return int(tmp), err
	}
}

func (DB2G db2g) toUInt(bytes []byte, column *column, conn *DmConnection) (uint, error) {
	if strconv.IntSize == 32 {
		tmp, err := DB2G.toUInt32(bytes, column, conn)
		return uint(tmp), err
	} else {
		tmp, err := DB2G.toUInt64(bytes, column, conn)
		return uint(tmp), err
	}
}

func (DB2G db2g) toFloat32(bytes []byte, column *column, conn *DmConnection) (float32, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}
		return float32(bytes[0]), nil
	case SMALLINT:
		return float32(util.Dm_build_586.Dm_build_681(bytes, 0)), nil
	case INT:
		return float32(util.Dm_build_586.Dm_build_686(bytes, 0)), nil
	case BIGINT:
		return float32(util.Dm_build_586.Dm_build_691(bytes, 0)), nil
	case REAL:
		return util.Dm_build_586.Dm_build_696(bytes, 0), nil
	case DOUBLE:
		dval := util.Dm_build_586.Dm_build_700(bytes, 0)
		return float32(dval), nil
	case DECIMAL:

	case CHAR, VARCHAR2, VARCHAR, CLOB:

	}
	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toFloat64(bytes []byte, column *column, conn *DmConnection) (float64, error) {
	switch column.colType {
	case BIT, BOOLEAN, TINYINT:
		if bytes == nil || len(bytes) == 0 {
			return 0, nil
		}
		return float64(bytes[0]), nil
	case SMALLINT:
		return float64(util.Dm_build_586.Dm_build_681(bytes, 0)), nil
	case INT:
		return float64(util.Dm_build_586.Dm_build_686(bytes, 0)), nil
	case BIGINT:
		return float64(util.Dm_build_586.Dm_build_691(bytes, 0)), nil
	case REAL:
		return float64(util.Dm_build_586.Dm_build_696(bytes, 0)), nil
	case DOUBLE:
		return util.Dm_build_586.Dm_build_700(bytes, 0), nil
	case DECIMAL:

	case CHAR, VARCHAR2, VARCHAR, CLOB:

		tval, err := DB2G.charToFloat64(bytes, column, conn)
		if err != nil {
			return 0, err
		}
		return tval, nil
	}

	return 0, ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toDmBlob(value []byte, column *column, conn *DmConnection) *DmBlob {

	switch column.colType {
	case BLOB:
		return newBlobFromDB(value, conn, column, conn.lobFetchAll())
	}

	return nil
}

func (DB2G db2g) toDmClob(value []byte, conn *DmConnection, column *column) *DmClob {

	switch column.colType {
	case CLOB:
		return newClobFromDB(value, conn, column, conn.lobFetchAll())
	}

	return nil
}

func (DB2G db2g) toDmDecimal(value []byte, column *column, conn *DmConnection) (*DmDecimal, error) {

	switch column.colType {
	case DECIMAL:
		return decodeDecimal(value, int(column.prec), int(column.scale))
	}

	return nil, nil
}

func (DB2G db2g) toTime(bytes []byte, column *column, conn *DmConnection) (time.Time, error) {
	switch column.colType {
	case DATE, TIME, TIME_TZ, DATETIME_TZ, DATETIME:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		return toTimeFromDT(dt, int(conn.dmConnector.localTimezone)), nil
	case CHAR, VARCHAR2, VARCHAR, CLOB:
		return toTimeFromString(DB2G.charToString(bytes, column, conn), int(conn.dmConnector.localTimezone)), nil
	}
	return time.Now(), ECGO_DATA_CONVERTION_ERROR.throw()
}

func (DB2G db2g) toObject(bytes []byte, column *column, conn *DmConnection) (interface{}, error) {

	switch column.colType {
	case BIT, BOOLEAN:
		return bytes[0] != 0, nil

	case TINYINT:

		return util.Dm_build_586.Dm_build_677(bytes, 0), nil
	case SMALLINT:
		return util.Dm_build_586.Dm_build_681(bytes, 0), nil
	case INT:
		return util.Dm_build_586.Dm_build_686(bytes, 0), nil
	case BIGINT:
		return util.Dm_build_586.Dm_build_691(bytes, 0), nil
	case DECIMAL:

	case REAL:
		return util.Dm_build_586.Dm_build_696(bytes, 0), nil
	case DOUBLE:
		return util.Dm_build_586.Dm_build_700(bytes, 0), nil
	case DATE, TIME, DATETIME, TIME_TZ, DATETIME_TZ:
		dt := decode(bytes, column.isBdta, int(column.colType), int(column.scale), int(conn.dmConnector.localTimezone), int(conn.DbTimezone))
		return toTimeFromDT(dt, int(conn.dmConnector.localTimezone)), nil
	case BINARY, VARBINARY:
		return bytes, nil
	case BLOB:
		blob := newBlobFromDB(bytes, conn, column, conn.lobFetchAll())

		if util.StringUtil.EqualsIgnoreCase(column.typeName, "LONGVARBINARY") {

			l, err := blob.GetLength()
			if err != nil {
				return nil, err
			}
			return blob.getBytes(1, int32(l))
		} else {
			return blob, nil
		}
	case CHAR, VARCHAR, VARCHAR2:
		val := DB2G.charToString(bytes, column, conn)
		if isBFile(int(column.colType), int(column.prec), int(column.scale)) {

		}

		return val, nil
	case CLOB:
		clob := newClobFromDB(bytes, conn, column, conn.lobFetchAll())
		if util.StringUtil.EqualsIgnoreCase(column.typeName, "LONGVARCHAR") {

			l, err := clob.GetLength()
			if err != nil {
				return nil, err
			}
			return clob.getSubString(1, int32(l))
		} else {
			return clob, nil
		}
	case INTERVAL_YM:
		return newDmIntervalYMByBytes(bytes), nil
	case INTERVAL_DT:
		return newDmIntervalDTByBytes(bytes), nil
	case ARRAY:
		return TypeDataSV.bytesToArray(bytes, nil, column.typeDescriptor)
	case SARRAY:
		return TypeDataSV.bytesToSArray(bytes, nil, column.typeDescriptor)
	case CLASS:

	case PLTYPE_RECORD:

	default:
		return nil, ECGO_DATA_CONVERTION_ERROR.throw()
	}

	return nil, ECGO_DATA_CONVERTION_ERROR.throw()
}
